#include "AnimatedSprite.h"

AnimatedSprite::AnimatedSprite()
{
   setX(0);
   setY(0);
	currentFrame = 0;
	animationTimer.setMode(THIRTY_FRAMES);
    numLoops = -1;
	animationTimer.setLimit(1);
	animationTimer.storeTime();

	this->setUseHardware(false);	//	Disable HW support by default since it introduces transparency issues (alpha not supported)
	this->setUseRotoZoom(false);	//	Disable rotozoom by default since it is slower...
}

AnimatedSprite::AnimatedSprite(int x, int y)
{
	currentFrame = 0;
	animationTimer.setMode(THIRTY_FRAMES);
	numLoops = -1;
	animationTimer.setLimit(1);
	animationTimer.storeTime();
	setX(x);
   setY(y);
	this->setUseHardware(false);	//	Disable HW support by default since it introduces transparency issues (alpha not supported)
	this->setUseRotoZoom(false);	//	Disable rotozoom by default since it is slower...
}

AnimatedSprite::~AnimatedSprite()
{
	image.clear();
}

bool AnimatedSprite::hasCollided(AnimatedSprite &spr)
{
   uint a = image.getWidth()/2;
   uint b = spr.image.getWidth()/2;

   IntVector2d temp = *this - IntVector2d(spr.getX(),spr.getY());

	if (temp.lengthSquared() <= a * a + b * b)
	 	return true;

	return false;
}

bool AnimatedSprite::loadFrame(string fileName)
{
	return image.loadImage(fileName);
}

void AnimatedSprite::render(SDL_Surface *screen)
{
	image.renderImage(currentFrame,screen,getX(),getY());
}

void AnimatedSprite::update()
{
	animationTimer.update();
	if(animationTimer.getRemaining() < 0)
	{
		currentFrame++;
		animationTimer.setLimit(1);
	}
	if(currentFrame > image.size()-1)
	{
		if(numLoops == -1)
		{
			currentFrame = 0;
		}
		else if( numLoops > 0)
		{
		    currentFrame = 0;
		    numLoops--;
		}
		else
		{
			currentFrame = this->image.size()-1;
		}
	}
}

