#ifndef ANIMATEDSPRITE_H
#define ANIMATEDSPRITE_H

#include "IntVector2d.h"
#include "Image.h"
#include "Timer.h"

class AnimatedSprite : public IntVector2d
{
public:
   AnimatedSprite();
   AnimatedSprite(int x,int y);
	~AnimatedSprite();

   bool loadFrame(string fileName);     // adds a frame of animation for this sprite

   void setAlpha(uint alpha)
   {
      image.setAlpha(alpha);
   }
    uint getAlpha()
    {
       return image.getAlpha();
   }
   void render(SDL_Surface* screen);         // Draws the image onto the screen
	void update();
   bool hasCollided(AnimatedSprite &spr);		//	Has this sprite collided with the passed in sprite?

   uint getImageWidth()
   {
      return image.getWidth();
   }
   uint getImageHeight()
   {
      return image.getHeight();
   }
	void setFrameRate(uint fps)
	{
		animationTimer.setMode(fps);
	}
	void setLooping(int numLoops)
	{
	    this->numLoops = numLoops;
	}
	void setLooping(bool shouldLoop)
	{
		if(shouldLoop)
            numLoops = -1;
        else
            numLoops = 0;
	}
	void clearFrames()
	{
		image.clear();
	}
	void setUseHardware(bool useHW)
	{
		image.setUseHardware(useHW);
	}
	void setUseRotoZoom(bool useRZ)
	{
		image.setUseRotoZoom(useRZ);
	}
	void setZoom(float zoom)
	{
		image.setZoom(zoom);
	}
	void setScaleX(float scaleX)
	{
		image.setScaleX(scaleX);
	}
	void setScaleY(float scaleY)
	{
		image.setScaleY(scaleY);
	}
	void setRotation(float angle)
	{
		image.setRotation(angle);
	}
	uint getCurrentFrame()
	{
	   return currentFrame;
	}
	bool hasFinished()
	{
	   if(currentFrame >= image.size()-1 && numLoops == 0)
         return true;

	   return false;
	}
	void rewind()
	{
	   currentFrame = 0;
	}
private:
	Image image;
	Timer animationTimer;
	int numLoops;
	uint currentFrame;
};
#endif	//	ANIMATEDSPRITE_H
