#include "Emitter.h"

Emitter::Emitter()
{
    init();
}

Emitter::~Emitter()
{
    deInit();
}

void Emitter::init()
{
    animSprite = NULL;
    sprite = NULL;
    randGen.randSeed();

    gravity = 0;
    colour.setColour(WHITE);
    repeat = false;
    fadeColour = false;
    fadeValue = 0;
    gravity = 0;
    deviance = 0;
    lifeTime = 10;
    max = 1000;
    maxVelocity.setXY(6,6);
    position.setXY(0,0);
    upBoundary.setXY(320,240);
    dnBoundary.setXY(0,0);
    finished = false;
}

void Emitter::deInit()
{
    //  Tidy up sprite objects if used...
    if(sprite)
    {
        delete sprite;
        sprite = NULL;
    }
    if(animSprite)
    {
        delete animSprite;
        animSprite = NULL;
    }
    particles.clear();
}

void Emitter::render(SDL_Surface* screen)
{

    Image().screenLock(screen);
    //  Run through all particles
    uint iMax = particles.size();
    for(uint i = 0; i < iMax; i++)
    {
        if(animSprite)
        {
            particles[i].render(*animSprite, screen);
        }
        else if(sprite)
        {
            particles[i].render(*sprite,screen);
        }
        else
        {
            particles[i].render(screen);
        }
    }
    Image().screenUnlock(screen);
}

void Emitter::setLifeTime(int ms)
{
    lifeTime = ms;
}

void Emitter::setPosition(IntVector2d pos)
{
    position = pos;
}

void Emitter::update()
{
    uint iMax = particles.size();
    if(!particles[iMax-1].getLifeTime())
        finished = true;

    if(animSprite)
        animSprite->update();

    for(uint i = 0; i < iMax; i++)
    {
        particles[i].update();

        if(fadeColour)
        {
            if(animSprite)
            {
                //  Fade alpha instead of colour
                particles[i].changeAlpha(fadeValue,*animSprite);
            }
            else if(sprite)
            {
                //  Fade alpha instead of colour
                particles[i].changeAlpha(fadeValue,*sprite);
            }
            else
                particles[i].changeColour(fadeValue);
        }

        //  If this particle is dead, we create a new particle
        if(!particles[i].getLifeTime() && repeat)
        {
            reset(i);
        }
        else if(particles[i].getPosition().getX() < dnBoundary.getX() || particles[i].getPosition().getX() > upBoundary.getX()
        || particles[i].getPosition().getY() < dnBoundary.getY() || particles[i].getPosition().getY() > upBoundary.getY())
        {
            reset(i);
        }
    }
}

void Emitter::setMax(int max)
{
    max = max;
    particles.resize(max);
}
