#ifndef EMITTER_INCLUDED
#define EMITTER_INCLUDED

#include "Particle.h"
#include <vector>

using namespace std;

class Emitter
{
    public:
        Emitter();
        ~Emitter();
        void init();
        void deInit();

        bool loadSprite(string filename)
        {
            if(sprite)
                return false;

            sprite = new Sprite;
            sprite->setUseHardware(true);
            return sprite->setSpriteImage(filename);
        }
        bool loadAnimatedSprite(string filename)
        {
            if(animSprite)
                return false;

            animSprite = new AnimatedSprite;
            animSprite->setUseHardware(true);
            animSprite->setFrameRate(FIFTEEN_FRAMES);
            animSprite->setLooping(true);
            return animSprite->loadFrame(filename);
        }

        void render(SDL_Surface* screen);       //  Render all particles
        void update();                          //  Update all contained particles

        void setMax(int numParticles);          //  Sets the number of particles this emitter can emit
        void setLifeTime(int ms);               //  Sets how long a particle should be active.

        IntVector2d getPosition()
        {
            return position;
        }
        void setPosition(IntVector2d pos);      //  Sets the emiters position
        void addParticle()
        {
            particles.push_back(Particle(position));
        }
        void setMaxVelocity(IntVector2d vel)
        {
            maxVelocity = vel;
        }
        void setColour(Colour col)
        {
            colour = col;
        }
        void shouldFadeColour(bool shouldFade)
        {
            fadeColour = shouldFade;
        }
        void setFadeValue(int value)
        {
            fadeValue = value;
        }
        void setGravity(int g)
        {
            gravity = g;
        }
        void setDeviation(int d)
        {
            deviance = d;
        }
        void setLooping(bool l)
        {
            repeat = l;
        }
        bool isFinished()
        {
            return finished;
        }
        void reset(int i)
        {
            particles[i].setPosition(position);
            particles[i].setLifeTime(lifeTime);
            randGen.setLimits(0,maxVelocity.getX()*2);
            int x = randGen.nextInt()-maxVelocity.getX();
            randGen.setLimits(0,maxVelocity.getY()*2);
            int y = randGen.nextInt()-maxVelocity.getY();
            particles[i].setVelocity(IntVector2d(x,y));
            particles[i].setColour(colour);
            particles[i].setGravity(gravity);
            particles[i].setDeviation(deviance);
        }
        void resetPlayback()
        {
            uint iMax = particles.size();
            for(uint i = 0; i < iMax; i++)
                reset(i);

            finished = false;
        }
        void setUpBoundary(IntVector2d bound)
        {
            upBoundary = bound;
        }
        void setDownBoundary(IntVector2d bound)
        {
            dnBoundary = bound;
        }
        void setBoundaries(IntVector2d lowBound, IntVector2d hiBound)
        {
            setUpBoundary(hiBound);
            setDownBoundary(lowBound);
        }
    private:
        vector <Particle> particles;
        IntVector2d position;

        Sprite* sprite;
        AnimatedSprite* animSprite;

        uint max;                   //  The max number of particles
        int lifeTime;               //  The life of each particle
        int gravity;                //  The gravity of this particle system
        int deviance;               //  The deviance of each particle
        Colour colour;              //  The base colour of the particles
        IntVector2d maxVelocity;    //  The maximum velocity that each particle can travel at
        Random randGen;
        bool fadeColour;            //  Do we want the colour to gradually fade each cycle?
        bool repeat;                //  Should the particles repeat?
        bool finished;
        int fadeValue;              //  The rate that the colour fades
        IntVector2d upBoundary;     //  The bounding area that the paticle is permit to travel to.
        IntVector2d dnBoundary;     //  The lower boundary that the particle is permitted to travel to.
};

#endif // EMITTER_INCLUDED
