#include "Engine.h"

Engine::Engine()
{
    //	Default constructor - As this engine was written from a GP2X viewpoint, things are setup in that regard.
    appName = "Default SDL Application";
    xRes = 320;
    yRes = 240;
    gameTimer.setMode(THIRTY_FRAMES);
    isPaused = false;
}
Engine::Engine(string appName, uint xRes, uint yRes, uint fps)
{
    this->appName = appName;
    this->xRes = xRes;
    this->yRes = yRes;
    gameTimer.setMode(fps);
    this->isPaused = false;
}
Engine::~Engine()
{
	TTF_Quit();
	Mix_CloseAudio();							// sutdown SDL_mixer
	SDL_FreeSurface(screen);	//	Delete screenbuffer
	SDL_Quit();		//	Shut down SDL tidyly
	#ifdef PLATFORM_GP2X
		MMUHack hack;
		hack.closeHack();
		if(loadMenu)	//	Quit to GP2X menu if required to do so by commandline.
		{
			chdir("/usr/gp2x");
			execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
		}
	#endif
}

void Engine::argHandler(int argc, char** argv)
{
    //	This is a default CMD handler with things in place to handle the needs of a GP2X
    if (argc <= 1) // check for commandline args.
    {
    // No cmd args...
    }
    else
    {
        // Do further processing
        for (int arg=1; arg<argc; arg++)
        {
            if ((argv[arg][0] == '-') || (argv[arg][0] == '/'))//look for escape characters
            {
                switch (argv[arg][1])//then look for arguments
                {
                    case 'r': //	Idication to return to the menu -r
                    {
                        //printf(argv[arg]+2 );
                        #ifdef PLATFORM_GP2X
                            loadMenu = true;	//	We should return to GP2X menu on program exit.
                        #endif
                    break;
                    }
                    default:
                    {
                        printf("Invalid Parameter: %s!",(char*)(argv[arg][1]));
                        break;
                    }
                }   //  End switch
            }   //  End if
        }   //  End for
    }   //  End if/else
}

void Engine::init()
{
    #ifdef PLATFORM_GP2X
        this->loadMenu = false;
    #endif
		  state = NULL;
		  state = new BaseState;

    if(SDL_Init(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER) < 0)			// Initialise SDL
	{
        fprintf (stderr, "Couldn't initialize SDL: %s\n", SDL_GetError ());
		exit (1);
	}

	/* Set xRes x yRes 16-bits video mode */
	screen = SDL_SetVideoMode (xRes, yRes, 16, SDL_SWSURFACE);
	if (screen == NULL)
	{
		fprintf (stderr, "Couldn't set %d x %d x 16 video mode: %s\n",xRes, yRes, SDL_GetError ());
		exit (2);
	}

    #ifdef PLATFORM_PC
        //  If we are not on the GP2X, we can display a window title.
        SDL_WM_SetCaption(this->appName.c_str(), 0);
    #endif
   	SDL_ShowCursor(SDL_DISABLE);															// Disable mouse cursor on gp2x

   // Setup timer now
	gameTimer.setLimit(1);	//	Set the specified fps for physics updates
	gameTimer.storeTime();

#ifdef PLATFORM_PC
	Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, AUDIO_S16, MIX_DEFAULT_CHANNELS, 4096);		// Initialize SDL_mixer for PC, buffer is set higher
#endif
#ifdef PLATFORM_GP2X
	MMUHack hack;
   hack.initHack();
	Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, AUDIO_S16, MIX_DEFAULT_CHANNELS, 128);			// Initialize SDL_mixer for GP2X, buffer is set lower than PC
#endif
	TTF_Init();				//	Startup font library
}

bool Engine::stateLoop()
{
   //  Check state for exit condition
   if(state->getNullifyState())
   {
		state->nullifyState();
      return false;  // End program execution
   }
   else if (state->getNeedInit() == false)
   {
      //  Update timer
      gameTimer.update();

      if(gameTimer.getRemaining() <= 0)
      {
         //  Update physics
         state->update();
         //  Take input
         state->keyboard();
         //  Render objects
         state->render(screen);
         gameTimer.setLimit(1);
      }
      else
      {
         SDL_Delay(1);  // Release CPU briefly
      }
      return true;   // Continue program execution
   }
   else
   {
		// check and change states
		//getVariables();
		state->clear();
		stateManagement();
		//setVariables();

      // Initialise the changed state
      state->init();
      state->setNeedInit(false);  // Set that we have performed the init
      return true;                  // Continue program execution
   }
   //  Should never reach here
   return false;
}
// OVERIDE this function in your game and check for your custom states
void Engine::stateManagement()
{
	//	Check if the state itself wants to change states
	uint next = state->getNextState();
	if(state)
	{
		delete state;
		state = NULL;
	}

   if(next == STATE_TITLE)
	{
		state = new StateTitle;
	}

}


void Engine::getVariables()
{
	variables.clear();
	uint iMax = (uint)state->variables.size();
	for(uint i = 0; i < iMax;i++)
		variables.push_back(state->variables[i]);
}


void Engine::setVariables()
{
	state->variables.clear();
	uint iMax = (uint)variables.size();
	for(uint i = 0; i < iMax;i++)
		state->variables.push_back(variables[i]);
}

