#ifndef ENGINE_H_INCLUDED
#define ENGINE_H_INCLUDED
/*
ABOUT: This is a template class for an SDL Games Engine, written by PokéParadox of Project Infinity.
You should not  have to modify this class directly when you use it. It is a base class and you
should write your own class BASED on this one using inheritence!

This engine provides abstractive layers to SDL functions and hopefully simplifies their operation.
It has been developed using DevkitGP2X and should work with no changes in that SDK.
You may have to modify things here and they if you use another IDE/SDK.

Since this is Pokéware... expect bugs... it's also nothing to do with me if it all goes wrong...

As far as I'm concerned you can do what you want with this. It's mainly intended for the homebrew
community.
*/
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_rotozoom.h>
//#include "SDL_gfx.h"
#include <SDL/SDL_mixer.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <vector>
//#include <fstream>
using namespace std;

//	Use MMU Hack for GP2X
#ifdef PLATFORM_GP2X
	#include "MMUHack.h"
#endif

// #include "Fixed.h"      // Fixed point support not ready yet
#include "Variable.h"
#include "States.h"
#include "Timer.h"         // Countdown timer

class Engine
{
   public:
      Engine();
      //  Construct app with this name, resolution and fps timing.
      Engine(string appName, uint xRes, uint yRes, uint fps);
      virtual ~Engine();

      //	Following functions can be overidden from default behaviour
      virtual void argHandler(int argc, char **argv);	//	Takes commandline args from main function for parsing.
      virtual void init();							//	All initialisation performed here.

      virtual void stateManagement();        // This function MUST be overided

      bool stateLoop();									//	Main game loop.

   protected:
      #ifdef PLATFORM_GP2X		//	GP2X Specific variables
         bool loadMenu;
      #endif

		//	The following functions are a means to pass variables between states
		void setVariables();	//	Sets the values in the new state
		void getVariables();	//	Gets the values from the state before it is changed
		vector<Variable> variables;
		//	The actual variable queues are stored in the states themselves
		//	We buffer the values between states

      string appName;
      uint xRes;
      uint yRes;
      bool isPaused;

      Timer gameTimer;
      BaseState* state;        // The state which changes
      SDL_Surface *screen;    //	The SDL surface we draw things to.
};

#endif // ENGINE_H_INCLUDED
