#ifndef IMAGE_H_INCLUDED
#define IMAGE_H_INCLUDED

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_rotozoom.h>
#include <string>
#include <vector>
using namespace std;
#include "Types.h"
#include "Colour.h"

// TODO: loading tilesheets...
// getPixel routines

class Image
{
   public:
      Image();
      ~Image();

		void loadImage(SDL_Surface* s)
		{
			images.push_back(s);
			isConverted.push_back(false);
		}
		bool loadImage(string imageName);                                 // Load an image into the vector
      //void loadImageSheet(string imageName,uint xTiles,uint yTiles);  // store the separated images on the vector

      void renderImage(SDL_Surface* dstimg,int destX, int destY);            // Render the default image
      void renderImage(uint i,SDL_Surface *dstimg, int destx, int desty);    // Render specific image to the surface

      uint size();                  // Returns the number of images in the vector
      void clear();                 // Clears all images out of the vector

      int getX();
      int getY();
      uint getAlpha();  // Presumes all images are same dimensionality.
      uint getWidth();
      uint getHeight();

		//	Set if to use a HW surface to blit
		void setUseHardware(bool useHW)
		{
			//	use Magenta as transparent colour if using hardware surfaces (does not support Alpha)
			//	Also HW surfaces are calculated on image load so you need to set this to true BEFORE loading an image.
			useHardware = useHW;
		}
		void setUseRotoZoom(bool useRZ)
		{
			useRotation = useRZ;
		}
		void setZoom(float zoom)
		{
			this->scaleX = zoom;
			useRotation = true;
		}
		void setScaleX(float scaleX)
		{
			this->scaleX = scaleX;
			useRotation = true;
		}
		void setScaleY(float scaleY)
		{
			this->scaleY = scaleY;
			useRotation = true;
		}
		void setRotation(float angle)
		{
			this->angle = angle;
			useRotation = true;
		}
      void setX(int x);
      void setY(int y);
      void setAlpha(uint alpha);

      // Pixel manipulation functions
      // check and lock screen
      void screenLock(SDL_Surface* screen);
      // set a pixel
      void setPixel(SDL_Surface *dstimg, int x, int y, Colour colour);
      void setPixel(SDL_Surface *dstimg, int x, int y, uint r, uint g, uint b,uint a);
      void setPixel(SDL_Surface* dstimg, int x, int y, uint r, uint g, uint b);
      // get a pixel
      Colour getPixel(SDL_Surface* img, int x, int y);
      uint* getPixelArray(SDL_Surface* img, int x, int y);
      // check and unlock screen
      void screenUnlock(SDL_Surface *screen);

   private:
		bool useHardware;
		bool useRotation;
      int x;
      int y;
		float scaleX;
		float scaleY;
		float angle;

      uint alpha;
      vector<SDL_Surface*>images;
		vector<bool>isConverted;
};

#endif // IMAGE_H_INCLUDED
