#include "InputHandler.h"
InputHandler::InputHandler()
{
   /* Check and open joystick device */
	if (SDL_NumJoysticks() > 0) {
		joy = SDL_JoystickOpen(0);
		if(!joy) {
			fprintf (stderr, "Couldn't open joystick 0: %s\n", SDL_GetError ());
		}
	}
    ResetKeys();
}


InputHandler::~InputHandler(){};


void InputHandler::Update()
{
    while (SDL_PollEvent(&mEvent))
    {
        switch(mEvent.type)
        {
#ifdef PLATFORM_GP2X
            case SDL_JOYBUTTONDOWN: KeyDown(mEvent.jbutton.button); break;
            case SDL_JOYBUTTONUP: KeyUp(mEvent.jbutton.button); break;
#endif
#ifdef PLATFORM_PC
            case SDL_KEYDOWN: KeyDown(mEvent.key.keysym.sym); break;
            case SDL_KEYUP: KeyUp(mEvent.key.keysym.sym); break;
#endif
        }
    }
}


#ifdef PLATFORM_GP2X
void InputHandler::KeyDown(const Uint8& key)
#endif
#ifdef PLATFORM_PC
void InputHandler::KeyDown(const SDLKey& key)
#endif
{
    switch(key)
    {
        case GP2X_START:     mKStart=true; break;
        case GP2X_SELECT:    mKSelect=true; break;
        case GP2X_UP:        mKUp=true; break;
        case GP2X_DOWN:      mKDown=true; break;
        case GP2X_LEFT:      mKLeft=true; break;
        case GP2X_RIGHT:     mKRight=true; break;
        case GP2X_UPLEFT:    mKUpLeft=true; break;
        case GP2X_UPRIGHT:   mKUpRight=true; break;
        case GP2X_DOWNLEFT:  mKDownLeft=true; break;
        case GP2X_DOWNRIGHT: mKDownRight=true; break;
        case GP2X_A:         mKA=true; break;
        case GP2X_B:         mKB=true; break;
        case GP2X_X:         mKX=true; break;
        case GP2X_Y:         mKY=true; break;
        case GP2X_L:         mKL=true; break;
        case GP2X_R:         mKR=true; break;
        case GP2X_VOLUP:     mKVolumeUp=true; break;
        case GP2X_VOLDOWN:   mKVolumeDown=true; break;
        case GP2X_CLICK:	  mKClick=true; break;
        default: break;
    }
}


#ifdef PLATFORM_GP2X
void InputHandler::KeyUp(const Uint8& key)
#endif
#ifdef PLATFORM_PC
void InputHandler::KeyUp(const SDLKey& key)
#endif
{
    switch(key)
    {
        case GP2X_START:     mKStart=false; break;
        case GP2X_SELECT:    mKSelect=false; break;
        case GP2X_UP:        mKUp=false; break;
        case GP2X_DOWN:      mKDown=false; break;
        case GP2X_LEFT:      mKLeft=false; break;
        case GP2X_RIGHT:     mKRight=false; break;
        case GP2X_UPLEFT:    mKUpLeft=false; break;
        case GP2X_UPRIGHT:   mKUpRight=false; break;
        case GP2X_DOWNLEFT:  mKDownLeft=false; break;
        case GP2X_DOWNRIGHT: mKDownRight=false; break;
        case GP2X_A:         mKA=false; break;
        case GP2X_B:         mKB=false; break;
        case GP2X_X:         mKX=false; break;
        case GP2X_Y:         mKY=false; break;
        case GP2X_L:         mKL=false; break;
        case GP2X_R:         mKR=false; break;
        case GP2X_VOLUP:     mKVolumeUp=false; break;
        case GP2X_VOLDOWN:   mKVolumeDown=false; break;
        case GP2X_CLICK:	  mKClick=false; break;
        default: break;
    }
}


void InputHandler::ResetKeys()
{
    SDL_Event event;
    while(SDL_PollEvent(&event));
    mKStart=mKSelect=mKUp=mKDown=mKLeft=mKRight=mKUpLeft=mKUpRight=mKDownLeft=mKDownRight=mKA=mKB=mKX=mKY=mKL=mKR=mKVolumeUp=mKVolumeDown=mKClick=false;
}
