#include "Music.h"

Music::Music()
{
	music = NULL;
	numLoops = -1;	// default loop forever;
}

void Music::init()
{
	#ifdef PLATFORM_GP2X
		Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, AUDIO_S16, MIX_DEFAULT_CHANNELS, 128);			// Initialize SDL_mixer for GP2X, buffer is set lower than PC
	#endif

	#ifdef PLATFORM_PC
		Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, AUDIO_S16, MIX_DEFAULT_CHANNELS, 4096);		// Initialize SDL_mixer for PC, buffer is set higher
	#endif
}

Music::~Music()
{
	freeAll();
}

void Music::freeAll()
{
	if(music)
	{
		Mix_FreeMusic(music);				// Release the memory allocated to music
		music = NULL;
	}
}
void Music::shutdown()
{
	Mix_CloseAudio();							// sutdown SDL_mixer
}

bool Music::loadMusic(string musicFile)
{
	music = NULL;
	music = Mix_LoadMUS(musicFile.c_str());
	if(music)
		return true;
	return false;
}

bool Music::isPlaying()
{
	if(Mix_PlayingMusic())
		return true;
	return false;
}

bool Music::isPaused()
{
	if(Mix_PausedMusic())
		return true;
	return false;
}

void Music::pause()
{
	if(isPlaying())
		Mix_PauseMusic();
}

void Music::play()
{
	if(!isPaused())
		Mix_PlayMusic(music, numLoops);	// Play the music
	else
		Mix_ResumeMusic();
}

void Music::stop()
{
	Mix_HaltMusic();
}

void Music::fade(uint ms)
{
	Mix_FadeOutMusic((int)ms);
}

void Music::setLooping(bool looping)
{
	if(looping)
		numLoops = -1;
	else
		numLoops = 1;
}

void Music::setLooping(uint numLoops)
{
	this->numLoops = numLoops;
}

void Music::setVolume(uint volume)
{
	Mix_VolumeMusic(volume);
}
