#include "Parser.h"

Command Parser::getNextCommand()
{
	Command temp;

	if (!commandList.empty())
	{
		temp = commandList.front();
		commandList.pop();
	}

	return temp;
}

bool Parser::createCommandList(string fileName)
{
	//	open script file
	load(fileName);

	Command thisCommand;
	bool comment = false;

	string variableName;	//	Stores the variable name
	string variableValue;	//	stores the values
	string tempLine;		//	Stores the line for working with
	//	run through the file
	for(int i = 0; size(); i++)
	{
		//	Reset the comand
		thisCommand.commandType = PARSE_ERROR;
		thisCommand.params.clear();
		thisCommand.filename = "NONE";
		comment = false;

		tempLine = getLine(i);

		//	Get the variable name
		if(strstr(tempLine.c_str(), ":"))
		{
			variableName = getVariableName(tempLine);	//	Get the name of the variable
			tempLine = stripVariableName(tempLine);			//	Now leave only the values
		}
		else
		{
			variableName = "PARSE_ERROR";
		}
		//	Check for comments
		//	if we find the semi-colon at the beginning of the line we are done!
		if(tempLine[0] == ';')
		{
			comment = true;
		}

		//	Check the variable name for keywords
		unsigned int tempCom = thisCommand.getCommand(variableName);

        if (tempCom == SOUNDSET || tempCom == A || tempCom == B || tempCom == X || tempCom == Y
        || tempCom == L || tempCom == R || tempCom == CLICK || tempCom == DUP || tempCom == DDOWN || tempCom == MUSIC
        || tempCom == END_SCRIPT)
        {
                thisCommand.commandType = tempCom;
                thisCommand.filename = getValue(tempLine);
                tempLine = stripValue(tempLine);
        }
        comment = true;

		//	Add command to queue
		if (thisCommand.commandType != PARSE_ERROR && thisCommand.commandType != END_SCRIPT)
			commandList.push(thisCommand);
		//	don't try to strip more than we have
		if (!comment)
			tempLine = stripValue(tempLine);
		//	Check to see if we have done parsing
		if(thisCommand.commandType == END_SCRIPT)
			return true;
	}
	return false;
}

string Parser::getVariableName(string line)
{
	char outString[1000];
	unsigned int x =0 ;
	while( x < strlen(line.c_str()))
	{
		if(line[x] != ':')
		{
			outString[x] = line[x];
		}
		else
		{
			break;
		}
		x++;
	}
	outString[x] = 0;
	line = outString;
	return line;
}

string Parser::stripVariableName(string line)
{
	char outString[1000];
	int x = 0;

	//	get to point where variable name stops
	while(line[x] != ':')
	{
		x++;
	}
	x++;

	//	Copy string accross
	unsigned int i = 0 ;
	while( i < strlen(line.c_str()))
	{
		if(line[x] == ';')
		{
			line[x+1] = 0;
		}
		else if(line[x] == 0)
		{
			break;
		}

		outString[i] = line[x];
		x++;i++;
	}
		outString[i] = 0;
		return outString;
}

string Parser::getParentDirectory(string fileName)
{
	unsigned int x = (unsigned int)strlen(fileName.c_str()) ;
	while( x > 0)
	{
		if(fileName[x] == '/')	//Check for furthest forwardslash
		{
			//fileName[x+1] = 0;	//chop off the filename
			break;
		}
		x--;
	}

	char output[1000];
	unsigned int i = 0;

	while ( i < x+1)
	{
		output[i] = fileName[i];
		i++;
	}
	output[i] = 0;
	fileName = output;
	return fileName;	//	Return just the path
}

string Parser::stripParentDirectory(string line)
{
	char outString[1000];
	int x = 0;

	//	get to point where variable stops
	while(line[x] != '/')
	{
		x++;
	}
	x++;

	//	Copy string accross
	unsigned int i = 0 ;
	while( i < strlen(line.c_str()))
	{
		if(line[x] == ';')
		{
			line[x+1] = 0;
		}
		else if(line[x] == 0)
		{
			break;
		}

		outString[i] = line[x];
		x++;i++;
	}
	outString[i] = 0;
	return outString;
}

string Parser::getValue(string var)
{
	char outString[1000];
	unsigned int x =0 ;
	while( x < strlen(var.c_str()))
	{
		if(var[x] != ';' && var[x] != ',')
		{
			outString[x] = var[x];
		}
		else
		{
			break;
		}
		x++;
	}
	outString[x] = 0;
	return outString;
}

string Parser::stripValue(string line)
{
	char outString[1000];
	int x = 0;

	//	get to point where variable stops
	while(line[x] != ',' && line[x] != ';')
	{
		x++;
	}
	x++;

	//	Copy string accross
	unsigned int i = 0 ;
	while( i < strlen(line.c_str()))
	{
		if(line[x] == ';')
		{
			line[x+1] = 0;
		}
		else if(line[x] == 0)
		{
			break;
		}

		outString[i] = line[x];
		x++;i++;
	}
	outString[i] = 0;
	return outString;
}

string Parser::getExtensionlessFilename(string line)
{
	char outString[1000];
	unsigned int x =0 ;
	while( x < strlen(line.c_str()))
	{
		if(line[x] != '.')
		{
			outString[x] = line[x];
		}
		else
		{
			break;
		}
		x++;
	}
	outString[x] = 0;
	line = outString;
	return line;
}
