#include "Particle.h"

Particle::Particle()
{
    init();
    setPosition(0,0);
}

Particle::Particle(int x, int y)
{
    init();
    setPosition(x,y);
}

Particle::Particle(IntVector2d vec)
{
    init();
    setPosition(vec);
}

Particle::~Particle()
{
    deInit();
}

void Particle::init()
{
    colour.setColour(WHITE);
    deviation = 0;
    gravity = 0;
    velocity.setXY(0,0);
    timer.storeTime();
    timer.setMode(MILLI_SECONDS);
    randGen.setLimits(-1,1);
    useDeviance = false;
}

void Particle::deInit()
{

}

void Particle::render(SDL_Surface* screen)
{
    Image().setPixel(screen, position.getX(), position.getY(), colour);
}

void Particle::render(AnimatedSprite &spr, SDL_Surface* screen)
{
    spr.setXY(position.getX(),position.getY());
    spr.render(screen);
}

void Particle::render(Sprite &spr, SDL_Surface* screen)
{
    spr.setXY(position.getX(),position.getY());
    spr.render(screen);
}

void Particle::update()
{
    if(timer.getRemaining())
    {
        timer.update();

        //  Update position of particle
        position = position + velocity;

        //  Apply gravity to verticle velocity
        velocity.setY(velocity.getY() + gravity);

        if(useDeviance)
        {
            position = position + IntVector2d(randGen.nextInt()-deviation,randGen.nextInt()-deviation);
        }
    }
}

void Particle::setColour(Colour col)
{
    colour = col;
}

void Particle::setColour(int r, int g, int b)
{
    colour = Colour(r,g,b);
}

void Particle::setDeviation(int dev)
{
    deviation = dev;
    randGen.setLimits(0,dev*2);
    useDeviance = true;
}

void Particle::setGravity(int g)
{
    gravity = g;
}

void Particle::setLifeTime(int ms)
{
    timer.setLimit(ms);
}

void Particle::setPosition(IntVector2d vec)
{
    position = vec;
}

void Particle::setPosition(int x, int y)
{
    position = IntVector2d(x,y);
}

void Particle::changeColour(int value)
{
    if(value < 0)
    {
        if(colour.red)
            colour.red += value;
        if(colour.green)
            colour.green += value;
        if(colour.blue)
            colour.blue += value;
    }
    else
    {
        if(colour.red < 255-value)
            colour.red += value;
        if(colour.green < 255-value)
            colour.green += value;
        if(colour.blue < 255-value)
            colour.blue += value;
    }
}

void Particle::changeAlpha(int value, Sprite& spr)
{
    uint newAlpha = spr.getAlpha() + value;

    if(newAlpha >= 0 || newAlpha <= 255)
        spr.setAlpha(newAlpha);
}

void Particle::changeAlpha(int value, AnimatedSprite& spr)
{
    uint newAlpha = spr.getAlpha() + value;

    if(newAlpha >= 0 || newAlpha <= 255)
        spr.setAlpha(newAlpha);
}
