#ifndef PARTICLE_H_INCLUDED
#define PARTICLE_H_INCLUDED

#include "Types.h"
#include "IntVector2d.h"
//#include "Fixed.h"
#include "AnimatedSprite.h"
#include "Sprite.h"
#include "Colour.h"
#include "Timer.h"
#include "Random.h"
#include <cmath>
#include "Image.h"

/*
This is a particle, it can use coloured pixels, animated sprites and static sprites for each particle.
The sprite object must be passed into the render function of the particle from the Emitter...
This is trying to avoid floats but may have to be extended to used Fixed point...
*/

class Particle
{
    public:
        Particle(IntVector2d vec);
        Particle(int x, int y);
        Particle();
        void init();
        void deInit();
        ~Particle();

        void update();

        //  WARNING THE EMITTER HANDLES SCREEN LOCKING AND UNLOCKING!
        void render(Sprite &spr,SDL_Surface* screen);
        void render(AnimatedSprite & spr, SDL_Surface* screen);
        void render(SDL_Surface* screen);

        void setColour(Colour col);
        void setColour(int r, int g, int b);
        void changeColour(int value);
        void changeAlpha(int value, Sprite& spr);
        void changeAlpha(int value, AnimatedSprite& spr);

        void setPosition(IntVector2d vec);
        void setPosition(int x, int y);

        IntVector2d getPosition()
        {
            return position;
        }

        void setDeviation(int dev);
        void setGravity(int g);
        void setLifeTime(int time_ms);
        int getLifeTime()
        {
            return timer.getRemaining();
        }
        void setVelocity(IntVector2d vel)
        {
            velocity = vel;
        }
    private:
        IntVector2d position;
        IntVector2d velocity;


        Colour colour;      // colour of the particle pixel used if no Sprite object is given
        int deviation;      //  This is the random variance of the particle position
        int gravity;        //  Should this partile be influenced by gravity?
        Timer timer;        //  This regulates how long this particle should live for.
        Random randGen;
        bool useDeviance;
};

#endif // PARTICLE_H_INCLUDED
