#include "Sound.h"

Sound::Sound()
{
	sound = NULL;
	channel = -1;
	loops = 0;
}

void Sound::init()
{
	#ifdef PLATFORM_GP2X
		Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, AUDIO_S16, MIX_DEFAULT_CHANNELS, 128);			// Initialize SDL_mixer for GP2X, buffer is set lower than PC
	#endif

	#ifdef PLATFORM_PC
		Mix_OpenAudio(MIX_DEFAULT_FREQUENCY, AUDIO_S16, MIX_DEFAULT_CHANNELS, 4096);		// Initialize SDL_mixer for PC, buffer is set higher
	#endif
}

void Sound::shutdown()
{
	Mix_CloseAudio();																// Close SDL_Mixer Audio
}

Sound::~Sound()
{
	freeAll();
}

void Sound::freeAll()
{
	if(sound)
	{
		Mix_FreeChunk(sound);															// Release the memory allocated to sound
		sound = NULL;
	}
}

bool Sound::loadSound(std::string soundName)
{
	if(channel != -1)
	{
		channel = -1;
	}
	sound = NULL;
	sound = Mix_LoadWAV(soundName.c_str());										// load sound.wav from sdcard

	if(sound)
		return true;
	return false;
}

bool Sound::isPaused()
{
	if(Mix_Paused(channel))
		return true;
	return false;
}

bool Sound::isPlaying()
{
	if(Mix_Playing(channel))
		return true;
	return false;
}

void Sound::play()
{
	if(isPaused())
		Mix_Resume(channel);
	else if(channel == -1 || !isPlaying())
		channel = Mix_PlayChannel(channel, sound, loops);
}

void Sound::pause()
{
	if(isPlaying())
		Mix_Pause(channel);
}

void Sound::stop()
{
	Mix_HaltChannel(channel);
}
