#include "StateTitle.h"

StateTitle::StateTitle()
{
    nullify = false;
    maxSets = 0;
}

StateTitle::~StateTitle()
{
	clear();
}

void StateTitle::clear()
{
	for(uint s = 0; s < maxSets; s++)
    {
        if(hasMusic[s])
            music[s].freeAll();

        for(uint t = 0 ; t < 9; t++)
        {
            if(hasSound[s][t])
            {
                soundSets[s][t].freeAll();
                soundNames[s][t].clear();
            }
        }
        soundSets[s].clear();
        soundNames[s].clear();
    }
    soundSets.clear();
    music.clear();
    hasMusic.clear();
    names.clear();
    songNames.clear();
    soundNames.clear();
}

void StateTitle::loadSoundSet(string fileName)
{
    static const string dir = "sounds/";
    Parser parser;
    if(parser.createCommandList(fileName))
    {
        Command tempCom;
        do  //  We process the command list until there is an error
        {
            tempCom = parser.getNextCommand();
            if(tempCom.commandType == MUSIC)
            {
                if(music[setNum].loadMusic(dir+names[setNum]+"/"+tempCom.filename))
                {
                    hasMusic[setNum] = true;
                    songNames[setNum] = tempCom.filename;
                }
            }
            else if (tempCom.commandType == A)
            {
                if(soundSets[setNum][0].loadSound(dir+names[setNum]+"/"+tempCom.filename))
                {
                    hasSound[setNum][0] = true;
                    soundNames[setNum][0] = tempCom.filename;
                }
            }
            else if (tempCom.commandType == B)
            {
                if(soundSets[setNum][1].loadSound(dir+names[setNum]+"/"+tempCom.filename))
                {
                    hasSound[setNum][1] = true;
                    soundNames[setNum][1] = tempCom.filename;
                }
            }
            else if (tempCom.commandType == X)
            {
                if(soundSets[setNum][2].loadSound(dir+names[setNum]+"/"+tempCom.filename))
                {
                    hasSound[setNum][2] = true;
                    soundNames[setNum][2] = tempCom.filename;
                }
            }
            else if (tempCom.commandType == Y)
            {
                if(soundSets[setNum][3].loadSound(dir+names[setNum]+"/"+tempCom.filename))
                {
                    hasSound[setNum][3] = true;
                    soundNames[setNum][3] = tempCom.filename;
                }
            }
            else if (tempCom.commandType == L)
            {
                if(soundSets[setNum][4].loadSound(dir+names[setNum]+"/"+tempCom.filename))
                {
                    hasSound[setNum][4] = true;
                    soundNames[setNum][4] = tempCom.filename;
                }
            }
            else if (tempCom.commandType == R)
            {
                if(soundSets[setNum][5].loadSound(dir+names[setNum]+"/"+tempCom.filename))
                {
                    hasSound[setNum][5] = true;
                    soundNames[setNum][5] = tempCom.filename;
                }
            }
            else if (tempCom.commandType == CLICK)
            {
                if(soundSets[setNum][6].loadSound(dir+names[setNum]+"/"+tempCom.filename))
                {
                    hasSound[setNum][6] = true;
                    soundNames[setNum][6] = tempCom.filename;
                }
            }
            else if (tempCom.commandType == DUP)
            {
                if(soundSets[setNum][7].loadSound(dir+names[setNum]+"/"+tempCom.filename))
                {
                    hasSound[setNum][7] = true;
                    soundNames[setNum][7] = tempCom.filename;
                }
            }
            else if (tempCom.commandType == DDOWN)
            {
                if(soundSets[setNum][8].loadSound(dir+names[setNum]+"/"+tempCom.filename))
                {
                    hasSound[setNum][8] = true;
                    soundNames[setNum][8] = tempCom.filename;
                }
            }
        }
        while (tempCom.commandType != PARSE_ERROR);
    }
    setNum++;
}

void StateTitle::parseSoundList()
{
    maxSets = setNum;
    setNum = 0;
    Parser parser;
    //  First we parse the SoundList.txt file
    if(parser.createCommandList("sounds/SoundList.txt"))
    {
        //  resize storage to hold the number of sets
        uint numCommands = parser.getNumCommands();
        music.resize(numCommands);
        songNames.resize(numCommands);
        soundSets.resize(numCommands,9);
        hasSound.resize(numCommands,9);
        soundNames.resize(numCommands,9);

        for(uint b = 0; b < numCommands; b++)
        {
            hasMusic.push_back(false);

            for(uint v = 0; v < 9; v++ )
                hasSound[b][v] = false;
        }

        Command tempCom;
        do  //  We process the command list until there is an error
        {
            tempCom = parser.getNextCommand();
            if(tempCom.commandType == SOUNDSET)
            {
                names.push_back(tempCom.filename);
                loadSoundSet("sounds/" + tempCom.filename + "/Sounds.txt");
            }
        }
        while (tempCom.commandType != PARSE_ERROR);
    }
}

void StateTitle::init()
{
    text.loadFont("font/bip.ttf",14);
    parseSoundList();
    setNum = 0;
}

void StateTitle::keyboard()
{

    // Perform input handling here
    input.Update();

    if (input.IsA() && hasSound[setNum][0])
    {
        soundSets[setNum][0].play();
    }
    if(input.IsB() && hasSound[setNum][1])
    {
        soundSets[setNum][1].play();
    }
    if(input.IsX() && hasSound[setNum][2])
    {
        soundSets[setNum][2].play();
    }
    if(input.IsY() && hasSound[setNum][3])
    {
        soundSets[setNum][3].play();
    }
    if(input.IsL() && hasSound[setNum][4])
    {
        soundSets[setNum][4].play();
    }
    if(input.IsR() && hasSound[setNum][5])
    {
        soundSets[setNum][5].play();
    }
    if(input.IsClick() && hasSound[setNum][6])
    {
        soundSets[setNum][6].play();
    }
    if(input.IsUp() && hasSound[setNum][7])
    {
        soundSets[setNum][7].play();
    }
    if (input.IsDown() && hasSound[setNum][8])
    {
        soundSets[setNum][8].play();
    }

    if(input.IsLeft() && setNum > 0)
    {
        setNum--;
        input.ResetKeys();
    }
    else if(input.IsRight() && setNum < maxSets)
    {
        setNum++;
        input.ResetKeys();
    }

    if(input.IsSelect() && hasMusic[setNum])
    {
        if(music[setNum].isPlaying())
            music[setNum].stop();
        else
            music[setNum].play();

        input.ResetKeys();
    }

    if(input.IsStart())
        this->nullifyState();

    //	adjust volume
    uint volume = music[0].getVolume();
    if(input.IsVolumeUp() && volume < 128)
        music[0].setVolume(volume+2);
    else if(input.IsVolumeDown() && volume > 0)
        music[0].setVolume(volume-2);

}

void StateTitle::render(SDL_Surface *screen)
{
   // Clear screen
	//	The clear colour here is hacked to show the splash image correctly...
	SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 0,0,0));

    text.setPosition(IntVector2d(160,0));
    text.setColour(Colour(WHITE));
    text.setRelativity(true);
    text.print(screen,"SoundBox\n");
    text.print(screen,"by PokeParadox\n");
    text.print(screen,"projectinfinity.org.uk\n\n");
    text.print(screen,names[setNum] + "\n");
    if(hasMusic[setNum])
        text.print(screen,"Music: " + songNames[setNum] + "\n");
    if(hasSound[setNum][0])
        text.print(screen,"A: " + soundNames[setNum][0]);
    if(hasSound[setNum][1])
        text.print(screen," B: " + soundNames[setNum][1] + "\n");
    if(hasSound[setNum][2])
        text.print(screen,"X: " + soundNames[setNum][2]);
    if(hasSound[setNum][3])
        text.print(screen," Y: " + soundNames[setNum][3] + "\n");
    if(hasSound[setNum][4])
        text.print(screen,"L: " + soundNames[setNum][4]);
    if(hasSound[setNum][5])
        text.print(screen," R: " + soundNames[setNum][5] + "\n");
    if(hasSound[setNum][6])
        text.print(screen,"CLICK: " + soundNames[setNum][6]);
    if(hasSound[setNum][7])
        text.print(screen," UP: " + soundNames[setNum][7] + "\n");
    if(hasSound[setNum][8])
        text.print(screen,"DOWN: " + soundNames[setNum][8] + "\n");
    //	Flush the cache on GP2X just before the screen is flipped
#ifdef PLATFORM_GP2X
    hack.flushCache(screen->pixels, (char*)screen->pixels + 76800);//320*240 = 76800
#endif
   //	Render screen.
   SDL_Flip(screen);
}

void StateTitle::update()
{
	//	set sound and music volume
	uint volume = music[0].getVolume();
	for(uint s = 0; s < maxSets; s++)
    {
        if(hasMusic[s])
            music[s].setVolume(volume);
        for(uint t = 0 ; t < 9; t++)
        {
            if(hasSound[s][t])
                soundSets[s][t].setVolume(volume);
        }
    }
}
