#include "TextHandler.h"

TextHandler::TextHandler()
{
	font = NULL;
	position.setXY(0,0);
	startPos.setXY(0,0);
//
}

void TextHandler::clear()
{
	if(font)
		TTF_CloseFont(font);			// Release the font object
}

TextHandler::~TextHandler()
{
	clear();
//								// Close SDL_TTF
}

bool TextHandler::loadFont(string fontName, int fontSize)
{
	font = NULL;
	font = TTF_OpenFont(fontName.c_str(), fontSize);
	if(font)
		return true;
	return false;
}

void TextHandler::print(SDL_Surface* screen, float number)
{
	print(screen,StringUtility().floatToString(number));
}

void TextHandler::print(SDL_Surface* screen, int number)
{

	print(screen,StringUtility().intToString(number));
}

void TextHandler::print(SDL_Surface* screen, string text)
{
	if(!text.size())
		return;

	print(screen,(char*)text.c_str());
}

void TextHandler::print(SDL_Surface* screen, char* text)
{
	uint CRcount = countCRs((string)text);
	if(CRcount)
	{
		string tString;
		tString = stripCRs((string)text).c_str();
		SDL_Surface* textSurface =  TTF_RenderText_Blended(font, tString.c_str(), colour);
		SDL_Rect textLocation = { position.getX(), position.getY(), 0, 0 };
		SDL_BlitSurface(textSurface, NULL, screen, &textLocation);
		textHeight = textSurface->h;
		textWidth = textSurface->w;
		if(relativePos)
		{
			if(position.getX() + textSurface->w > 320)
				position.setXY(startPos.getX(),position.getY() + textSurface->h);
			else if(CRcount)
			{
				for(uint i=0; i < CRcount; i++)
					position.setXY(startPos.getX(),position.getY() + textSurface->h);
			}
			else
			{
				position.setX(position.getX() + textSurface->w);
			}
		}
		SDL_FreeSurface(textSurface);
	}
	else
	{
		SDL_Surface* textSurface =  TTF_RenderText_Blended(font, text, colour);
		SDL_Rect textLocation = { position.getX(), position.getY(), 0, 0 };
		SDL_BlitSurface(textSurface, NULL, screen, &textLocation);
		textHeight = textSurface->h;
		textWidth = textSurface->w;
		if(relativePos)
		{
			if(position.getX() + textSurface->w > 320)
				position.setXY(startPos.getX(),position.getY() + textSurface->h);
			else
			{
				position.setX(position.getX() + textSurface->w);
			}
		}
		SDL_FreeSurface(textSurface);
	}
}

void TextHandler::setPosition(int x, int y)
{
	position.setXY(x,y);
	startPos.setXY(x,y);
}

void TextHandler::setPosition(IntVector2d position)
{
	this->position = position;
}

void TextHandler::setColour(Colour colour)
{
	this->colour = colour.getColour();
}

void TextHandler::setColour(uint red, uint green, uint blue)
{
	colour.r = red;
	colour.g = green;
	colour.b = blue;
}

void TextHandler::setRelativity(bool rel)
{
	relativePos = rel;
}

uint TextHandler::countCRs(string line)
{
	uint CRcount = 0;
	//	run through the string
	for(uint i = 0; i < (uint)line.size(); i++)
	{
		//	check for CR and increase CRcount if found
		if(line[i] == '\n')
			CRcount++;
	}
	return CRcount;
}

string TextHandler::stripCRs(string line)
{
	for(uint i = 0; i < line.size(); i++)
		if(line[i] == '\n')
			line[i] = 0;

	return line;
}
