#ifndef TEXTHANDLER_H
#define TEXTHANDLER_H

#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <string>
using namespace std;
#include "IntVector2d.h"
#include "Colour.h"
#include "StringUtility.h"
//Text will wrap after 58 characters, inc spaces and punctuation. (at size 12 pt)
//The text below shows how many characters can be used before the text will wrap.
///////////////////////////////////////////////////////////////////////////////////
//text.print(screen, "To be or not to be, that is the question Whether tis noble");

class TextHandler
{
public:
	TextHandler();		//	Initialise font handling
	~TextHandler();	//	Shutdown font handling

	bool loadFont(string fontName, int fontSize);		//	Loads a TTF

	void setPosition(IntVector2d position);				//	Sets the starting position of the text
	void setPosition(int x, int y);							//	Sets the starting position of the text
	void setColour(Colour colour);							//	Sets the font colour
	void setColour(uint red,uint green,uint blue);		//	Sets the font colour

	uint getTextHeight()
	{
		return textHeight;
	}
	uint getTextWidth()
	{
		return textWidth;
	}

	void setRelativity(bool);

//	You must have used the above functions in order to use the following print functions
	void print(SDL_Surface* screen, string text);		//	write a string to the screen
	void print(SDL_Surface* screen, char* text);			//	write a char* string to the screen
	void print(SDL_Surface* screen, int number);			//	write an integer to the screen
	void print(SDL_Surface* screen, float number);		//	write a float number to the screen

	void clear();
private:
	uint countCRs(string line);	//	Counts the number of carriage return symbols that are in the string.
	string stripCRs(string line);	//	Removes carriage returns from an input string
	TTF_Font* font;
	uint textHeight;
	uint textWidth;
	IntVector2d position;
	IntVector2d startPos;
	bool relativePos;
	SDL_Color colour;
};

#endif	//	TEXTHANDLER_H

