#ifndef TIMER_H_INCLUDED
#define TIMER_H_INCLUDED

#include <SDL/SDL.h>
#include "Types.h"

/*
#undef PLATFORM_PC
#define PLATFORM_GP2X
//*/

#ifdef PLATFORM_GP2X
//#include "Fixed.h"
#endif

#ifndef CLOCKS_PER_SEC
	#define CLOCKS_PER_SEC 1000
#endif

// TODO remove floats and replace with fixed point types... depends on fixed point types being ready...

enum TimerScalers
{
    //	Various modes that the timer can be put in.
    FIFTEEN_FRAMES,
    THIRTY_FRAMES,
    FIFTY_FRAMES,
    SIXTY_FRAMES,
    NANO_SECONDS,
    MICRO_SECONDS,
    MILLI_SECONDS,
    CENTI_SECONDS,
    DECI_SECONDS,
    SECONDS,
    MINUTES,
    HOURS
};

class Timer
{
   public:
      Timer()
      {
         time = 0;
         stored = 0;
         lastUpdate = SDL_GetTicks();
         mode = SIXTY_FRAMES;
         #ifdef PLATFORM_PC
         scaler = 0.0f;
         #endif
         #ifdef PLATFORM_GP2X
         /*
         scaler.setScaler(12);
         scaler.floatToFixed(0.0f);
         */
         scaler = 0.0f;
         #endif
      }
      Timer(int lim)
      {
         time = lim;
         stored = 0;
         lastUpdate = SDL_GetTicks();
         mode = SIXTY_FRAMES;
         #ifdef PLATFORM_PC
         scaler = 0.0f;
         #endif
         #ifdef PLATFORM_GP2X
         /*
         scaler.setScaler(12);
         scaler.floatToFixed(0.0f);
         */
         scaler = 0.0f;
         #endif
      }
      ~Timer(){};

      void setLimit(int timeLimit)    // Set countdown limit
      {
         time = timeLimit;
      }
      void setMode(uint mode)       // Set the timer scaler
      {
         this->mode = mode;
         calcScaler();
      }
      void incLimit(int changeOfLimit)// Change the current timelimit
      {
         time += changeOfLimit;
      }
      void storeTime()                // Store the current time
      {
         stored = time;
      }
      int getDifference()             // Return the difference between stored time and current time
      {
         return stored - time;
      }
      int getRemaining()              // Return the remaining coutdown time
      {
         return time;
      }
      void update()       // progress the timer.
      {
         #ifdef PLATFORM_GP2X
         /*
         Fixed temp(12);
         temp.assign((int)(SDL_GetTicks() - lastUpdate));
         if(temp.getValue() > scaler.getValue())  //one unit has passed
         */
         if(SDL_GetTicks() - lastUpdate > scaler)  //one unit has passed
         #endif
         #ifdef PLATFORM_PC
         if(SDL_GetTicks() - lastUpdate > scaler)  //one unit has passed
         #endif
         {
            lastUpdate = SDL_GetTicks();
            time--;
            //cout << "Timer: " << time << endl;
         }
      }

   private:
      void calcScaler();   // Calculate the resolution of timer updates
      uint mode;
      int time;
      int stored;
      int lastUpdate;
/*
#ifdef PLATFORM_GP2X
      Fixed scaler;
#endif

#ifdef PLATFORM_PC
      float scaler;
#endif
*/
    float scaler;
};
#endif // TIMER_H_INCLUDED
